Encoding: UTF-8
Package: stochvol
Type: Package
Title: Efficient Bayesian Inference for Stochastic Volatility (SV)
        Models
Version: 3.2.9
Authors@R: c(
	person("Darjus", "Hosszejni", role = c("aut", "cre"), email = "darjus.hosszejni@icloud.com", comment = c(ORCID = "0000-0002-3803-691X")),
	person("Gregor", "Kastner", role = c("aut"), email = "gregor.kastner@aau.at", comment = c(ORCID = "0000-0002-8237-8271")))
Description: Efficient algorithms for fully Bayesian estimation of stochastic volatility (SV) models with and without asymmetry (leverage) via Markov chain Monte Carlo (MCMC) methods. Methodological details are given in Kastner and Frühwirth-Schnatter (2014) <doi:10.1016/j.csda.2013.01.002> and Hosszejni and Kastner (2019) <doi:10.1007/978-3-030-30611-3_8>; the most common use cases are described in Hosszejni and Kastner (2021) <doi:10.18637/jss.v100.i12> and Kastner (2016) <doi:10.18637/jss.v069.i05> and the package examples.
License: GPL (>= 2)
Depends: R (>= 3.5)
Imports: Rcpp (>= 1.0), coda (>= 0.19), graphics, stats, utils,
        grDevices
Suggests: testthat (>= 3.0.0), mvtnorm, knitr
LinkingTo: Rcpp, RcppArmadillo (>= 0.9.900)
RoxygenNote: 7.3.2
BuildResaveData: best
VignetteBuilder: knitr
BugReports: https://github.com/gregorkastner/stochvol/issues
URL: https://gregorkastner.github.io/stochvol/
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-03-06 16:54:02 UTC; darjushosszejni
Author: Darjus Hosszejni [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3803-691X>),
  Gregor Kastner [aut] (ORCID: <https://orcid.org/0000-0002-8237-8271>)
Maintainer: Darjus Hosszejni <darjus.hosszejni@icloud.com>
Repository: CRAN
Date/Publication: 2026-03-07 06:10:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 12:27:18 UTC; unix
Archs: stochvol.so.dSYM
