# Generated by roxygen2: do not edit by hand

S3method("[",sv_multinormal)
S3method("[",svdraws)
S3method("[",svpredict)
S3method(as.array,svdraws)
S3method(as.array,svpredict)
S3method(density,sv_beta)
S3method(density,sv_constant)
S3method(density,sv_exponential)
S3method(density,sv_gamma)
S3method(density,sv_infinity)
S3method(density,sv_inverse_gamma)
S3method(density,sv_multinormal)
S3method(density,sv_normal)
S3method(logret,default)
S3method(mean,sv_beta)
S3method(mean,sv_constant)
S3method(mean,sv_exponential)
S3method(mean,sv_gamma)
S3method(mean,sv_infinity)
S3method(mean,sv_inverse_gamma)
S3method(mean,sv_multinormal)
S3method(mean,sv_normal)
S3method(paratraceplot,svdraws)
S3method(plot,svdraws)
S3method(plot,svpredict)
S3method(plot,svresid)
S3method(plot,svsim)
S3method(predict,svdraws)
S3method(print,summary.svdraws)
S3method(print,summary.svsim)
S3method(print,sv_beta)
S3method(print,sv_constant)
S3method(print,sv_exponential)
S3method(print,sv_gamma)
S3method(print,sv_infinity)
S3method(print,sv_inverse_gamma)
S3method(print,sv_multinormal)
S3method(print,sv_normal)
S3method(print,sv_priorspec)
S3method(print,svdraws)
S3method(print,svdraws_roll)
S3method(print,svpredict)
S3method(print,svsim)
S3method(range,sv_beta)
S3method(range,sv_constant)
S3method(range,sv_exponential)
S3method(range,sv_gamma)
S3method(range,sv_infinity)
S3method(range,sv_inverse_gamma)
S3method(range,sv_multinormal)
S3method(range,sv_normal)
S3method(rep,sv_normal)
S3method(rep.int,sv_normal,rep_int_sv_normal)
S3method(rep_len,sv_normal)
S3method(residuals,svdraws)
S3method(summary,svdraws)
S3method(summary,svdraws_roll)
S3method(summary,svsim)
export(default_fast_sv)
export(get_default_fast_sv)
export(get_default_general_sv)
export(latent)
export(latent0)
export(logret)
export(para)
export(paradensplot)
export(paratraceplot)
export(predlatent)
export(predvola)
export(predy)
export(priors)
export(runtime)
export(sampled_parameters)
export(specify_priors)
export(sv_beta)
export(sv_constant)
export(sv_exponential)
export(sv_gamma)
export(sv_infinity)
export(sv_inverse_gamma)
export(sv_multinormal)
export(sv_normal)
export(svbeta)
export(svlm)
export(svlsample)
export(svlsample_roll)
export(svsample)
export(svsample2)
export(svsample_fast_cpp)
export(svsample_general_cpp)
export(svsample_roll)
export(svsim)
export(svtau)
export(svtlsample)
export(svtlsample_roll)
export(svtsample)
export(svtsample_roll)
export(thinning)
export(update_fast_sv)
export(update_general_sv)
export(update_regressors)
export(update_t_error)
export(updatesummary)
export(validate_and_process_expert)
export(vola)
export(volplot)
importFrom(Rcpp,sourceCpp)
importFrom(coda,effectiveSize)
importFrom(coda,mcmc)
importFrom(coda,mcmc.list)
importFrom(coda,mcpar)
importFrom(coda,nchain)
importFrom(coda,niter)
importFrom(coda,nvar)
importFrom(coda,traceplot)
importFrom(coda,varnames)
importFrom(grDevices,col2rgb)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.default)
importFrom(graphics,points)
importFrom(graphics,title)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,clusterSetRNGStream)
importFrom(parallel,makeCluster)
importFrom(parallel,makePSOCKcluster)
importFrom(parallel,mclapply)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(stats,.getXlevels)
importFrom(stats,IQR)
importFrom(stats,coefficients)
importFrom(stats,cov)
importFrom(stats,dbeta)
importFrom(stats,delete.response)
importFrom(stats,density)
importFrom(stats,dexp)
importFrom(stats,dgamma)
importFrom(stats,dnorm)
importFrom(stats,lm)
importFrom(stats,lowess)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.pass)
importFrom(stats,plot.ts)
importFrom(stats,ppoints)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,qqplot)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(stats,sd)
importFrom(stats,start)
importFrom(stats,time)
importFrom(stats,ts.plot)
importFrom(utils,flush.console)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(stochvol, .registration = TRUE)
